from abc import abstractmethod

import serial


class ReaderError(Exception):
    """This is raised if the AsyncReader encounters an exception"""


class AsyncReader:
    """This class can be used for collecting text output asynchronously, and grouping into lines."""

    def __init__(self) -> None:
        self.partial_line = ""
        self.lines = []
        self.text = ""

    def get_line(self):
        try:
            new_text = self._get_data()
        except serial.serialutil.SerialException as e:
            raise ReaderError(e)
        except UnicodeEncodeError as e:
            raise ReaderError(e)

        if not new_text:
            return None

        # Clean up string
        new_text = new_text.replace("\r", "")
        new_text = new_text.replace("\0", "")

        lines = new_text.splitlines(keepends=True)

        lines[0] = self.partial_line + lines[0]

        # Save last line
        if lines[-1][-1] != "\n":
            self.partial_line = lines.pop()
        else:
            self.partial_line = ""

        self.lines = lines

        if self.lines:
            return self.lines.pop(0).strip()

    def get_accumulate(self):
        """This function will accumulate received text,
        until clear_accumulate is called."""

        new_text = self._get_data()
        if new_text:
            self.text += new_text
        return self.text

    def clear_accumulate(self):
        self.text = ""

    @abstractmethod
    def _get_data(self):
        pass


class AsyncReaderUART(AsyncReader):
    """Subclass with support for reading from a UART"""

    def __init__(self, serial):
        super().__init__()
        self.serial = serial

    # def get_line(serial):
    #     """
    #     Raises: serial.serialutil.SerialException, UnicodeDecodeError, InvalidOutput
    #     """

    #     # Some text was received
    #     _PARTIAL_STR += uart_str

    #     # print("partial_str:", partial_str)

    #     # Check if it's a full line now
    #     if _PARTIAL_STR[-1] == "\n" and len(_PARTIAL_STR) > 1:
    #         # Remove new line
    #         msg = _PARTIAL_STR.strip()
    #         read_line_clear()

    #         # Process this line
    #         m = re.match("(" + "|".join(Severity.list()) + "): (.*)", msg)
    #         if not m:
    #             raise SerialInvalidOutput(msg)

    #         if m.group(1) in Severity.list():
    #             severity = Severity(m.group(1))
    #             return (severity, m.group(2).strip())
    #         else:
    #             # Should not be reachable
    #             assert False

    #     raise SerialNoFullLine

    def get_data(self):
        return self.serial.read_until()


class AsyncReaderSocket(AsyncReader):
    """Subclass with support for reading from a socket."""

    def __init__(self, socket):
        """Provide the already opened socket.  The socket will be put in non-blocking mode."""
        super().__init__()
        self.socket = socket
        self.socket.setblocking(0)

    def _get_data(self):
        try:
            return self.socket.recv(1024).decode("utf-8")
        except BlockingIOError:
            return None



#!/usr/bin/python3

import telnetlib
import time
import sys
import os
import re


TIMEOUT = 3.0


class NetbooterPort:
    def __init__(self, ip_address, port, timeout=3.0):
        self._SLEEP_TIME = 1.0
        self.timeout = timeout
        self.ip_address = ip_address
        self.port = port

    def __str__(self):
        return f"{self.ip_address}:{self.port}"

    def reboot(self, port_num):
        tn = telnetlib.Telnet(self.ip_address, self.port, timeout=TIMEOUT)

        s = tn.read_some()
        time.sleep(self._self._SLEEP_TIME)

        s = ("rb " + str(port_num)).encode("ascii") + b"\r\n\r\n"
        tn.write(s)
        time.sleep(self._self._SLEEP_TIME)
        tn.close()

    def on(self, port_num):
        # print("On:", port_num)
        tn = telnetlib.Telnet(self.ip_address, self.port, timeout=TIMEOUT)

        s = tn.read_some()
        time.sleep(self._self._SLEEP_TIME)

        s = ("pset " + str(port_num) + " 1").encode("ascii") + b"\r\n\r\n"
        tn.write(s)
        time.sleep(self._self._SLEEP_TIME)
        tn.close()

    def off(self, port_num):
        # print("Off:", port_num)
        tn = telnetlib.Telnet(self.ip_address, self.port, timeout=TIMEOUT)

        s = tn.read_some()
        # print(s)
        time.sleep(self._self._SLEEP_TIME)

        s = ("pset " + str(port_num) + " 0").encode("ascii") + b"\r\n\r\n"
        tn.write(s)
        time.sleep(self._self._SLEEP_TIME)
        tn.close()

    def get_status(self):
        tn = telnetlib.Telnet(self.ip_address, self.port, timeout=TIMEOUT)

        s = tn.read_some()
        time.sleep(self._self._SLEEP_TIME)

        s = "pshow".encode("ascii") + b"\r\n"
        tn.write(s)
        time.sleep(self._self._SLEEP_TIME)

        s = ""
        while True:
            text = tn.read_eager()
            s += text.decode()
            if len(text) == 0:
                break

        tn.close()

        return s

    def is_on(self, port_num):
        text = self.get_status()
        lines = text.splitlines()

        for l in lines:
            m = re.match(r"\d+\|\s+Outlet" + str(port_num) + r"\|\s+(\w+)\s*\|", l.strip())
            if m:
                return m.group(1) == "ON"
        return None

    def print_usage():
        print("Incorrect usage!")
        print(os.path.splitext(__file__)[0], "<on|off|reboot|status> <port_num>")


def main():
    ip_address = "192.168.1.100"
    port = 23
    netbooter = NetbooterPort(ip_address, port)
    # Get command
    if len(sys.argv) < 2:
        netbooter.print_usage()
        return

    cmd = sys.argv[1].lower()
    if cmd == "status":
        print(netbooter.get_status())
        return

    if len(sys.argv) != 3:
        netbooter.print_usage()
        return

    if cmd not in ("on", "off", "reboot"):
        netbooter.print_usage()
        return

    port_num = sys.argv[2]

    if cmd == "on":
        netbooter.port_on(port_num)
    elif cmd == "off":
        netbooter.port_off(port_num)
    elif cmd == "reboot":
        netbooter.port_reboot(port_num)


if __name__ == "__main__":
    main()



import subprocess
import time

import argparse

import pyhubctl

from utils import error, print_color, TermColors

# TODO: At one point uhubctl failed with the following error:
# uhubctl: symbol lookup error: uhubctl: undefined symbol: libusb_free_container_id_descriptor
# I restarted the NUC and it worked, but I don't know what went wrong.
# That error was invisible to the other scripts...


class USBPortPower:
    def __init__(self, usb_phys_port):
        self.usb_location = ".".join(usb_phys_port.split(".")[:-1])
        self.usb_port = usb_phys_port.split(".")[-1]
        self.phc = pyhubctl.PyHubCtl()

    def cycle(self):
        self.phc.run(
            pyhubctl.Configuration(action="cycle", location=self.usb_location, ports=self.usb_port)
        )

    def off(self):
        self.phc.run(
            pyhubctl.Configuration(action="off", location=self.usb_location, ports=self.usb_port)
        )

    def on(self):
        self.phc.run(
            pyhubctl.Configuration(action="on", location=self.usb_location, ports=self.usb_port)
        )


def usbPowerCycle(usb_phys_port):

    usb_location = ".".join(usb_phys_port.split(".")[:-1])
    usb_port = usb_phys_port.split(".")[-1]
    print(" ".join(["uhubctl", "-a", "off", "-l", usb_location, "-p", usb_port]))
    proc_off = subprocess.run(
        ["uhubctl", "-a", "off", "-l", usb_location, "-p", usb_port],
        stdout=subprocess.DEVNULL,
        stderr=subprocess.DEVNULL,
    )
    time.sleep(1)
    proc_on = subprocess.run(
        ["uhubctl", "-a", "on", "-l", usb_location, "-p", usb_port],
        stdout=subprocess.DEVNULL,
        stderr=subprocess.DEVNULL,
    )
    time.sleep(1)

    if proc_off.returncode or proc_on.returncode:
        return False
    else:
        return True


def power_off(usb_phys_port, print_output=False):
    usb_location = ".".join(usb_phys_port.split(".")[:-1])
    usb_port = usb_phys_port.split(".")[-1]

    cmd = ["uhubctl", "-a", "off", "-l", usb_location, "-p", usb_port, "-r", "10"]
    proc_off = subprocess.run(cmd, capture_output=True, universal_newlines=True)

    if print_output:
        print(" ".join(cmd))
        print(proc_off.stdout)

    if proc_off.returncode:
        return False
    else:
        return True


def power_on(usb_phys_port, print_output=False):
    usb_location = ".".join(usb_phys_port.split(".")[:-1])
    usb_port = usb_phys_port.split(".")[-1]
    cmd = ["uhubctl", "-a", "on", "-l", usb_location, "-p", usb_port, "-r", "10"]
    proc_on = subprocess.run(cmd, capture_output=True, universal_newlines=True)

    if print_output:
        print(" ".join(cmd))
        print(proc_on.stdout)

    if proc_on.returncode:
        return False
    else:
        return True


def main():
    #  Power cycle UART USB port
    print("Power cycling UART")

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "usb_physical_port", help="The physical port of the USB (1-1.1 for example)"
    )

    args = parser.parse_args()

    result = usbPowerCycle(args.usb_physical_port)

    if True:
        print_color(TermColors.GREEN, "Success")
    else:
        error("USB power cycle failed")


if __name__ == "__main__":
    main()
    # usbPowerCycle("1-1.1")



from setuptools import setup

setup(
    name="yinstruments",
    packages=["yinstruments"],
    version="1.1.0",
    description="Experiment device control scripts for BYU's Configurable Computing Lab (https://ccl.byu.edu/)",
    author="Jeff Goeders",
    author_email="jeff.goeders@gmail.com",
    url="https://github.com/byuccl/yinstruments",
    install_requires=["pyudev", "serial"],
)



from pathlib import Path
import re
import subprocess

import pyudev


class USBFindError(Exception):
    pass


def find_dev_file_ttyUSB(usb_phys_port, interface):
    """This will find the correct /dev/ttyUSBX file for a given physical USB port and interface.

    Interface for Ultra96 (0 = jtag, 1 = uart)
    """

    match_str = match_str = "/devices/.*?/.*?/usb\d+/.*/(.*?)/ttyUSB\d+/tty/ttyUSB\d+$"
    return _find_dev_file(usb_phys_port, "USB", match_str, interface)


def find_dev_file_ttyACM(usb_phys_port, interface):
    """This will find the correct /dev/ttyACMX file for a given physical USB port and interface."""

    match_str = match_str = "/devices/.*?/.*?/usb\d+/.*/(.*?)/tty/ttyACM\d+$"
    return _find_dev_file(usb_phys_port, "ACM", match_str, interface)


def _find_dev_file(usb_phys_port, ttyType, match_str, interface):

    """This function will search through all /dev devices and find the one that maps to the given
    usb_phys_port, ttyType, regex match string and interface.

    This function works by running "udevadm info -q path -n /dev/ttyUSB0" for each serial port,
    and looking for the output that matches the location.  For example, if it is located at
    1-10.1, the output of interface 1 will be
    /devices/pci0000:00/0000:00:14.0/usb1/1-10/1-10.1/1-10.1:1.1/ttyUSB3/tty/ttyUSB3
    """

    # Iterate over all files in /dev
    match_found = None
    p = None
    for f in Path("/dev").iterdir():
        if re.match("/dev/tty" + ttyType + "\d+", str(f)):
            p = subprocess.run(
                ["udevadm", "info", "-q", "path", "-n", str(f)], stdout=subprocess.PIPE
            )
            # print(p.stdout.decode())
            # print(match_str)
            m = re.match(match_str, p.stdout.decode())
            if not m:
                raise USBFindError("Serial find regex error")
            if m.group(1) == (usb_phys_port + ":1." + str(interface)):
                if match_found is not None:
                    raise USBFindError(
                        "Multiple serial device matches for USB location "
                        + usb_phys_port
                        + ": "
                        + match_found
                        + " and "
                        + f
                    )
                match_found = f
                # print(p.stdout.decode())
            if not m:
                print(p.stdout.decode(), "did not match regex.")
                assert False
    if match_found is None:
        raise USBFindError("Could not find serial device at USB location " + usb_phys_port)

    return match_found


def find_dev_file_usb_bus(usb_phys_port):
    """
    For a given usb physical port, this function finds the USB bus device file.

    For example, a USB device at physical port 1-7.1 may be mapped to bus=3,device=7, which
    would mean that the associated usb bus device file would be
    /dev/bus/usb/003/007
    """
    context = pyudev.Context()
    devPath = None
    for device in context.list_devices(subsystem="tty"):
        if "DEVPATH" in device.properties and usb_phys_port in device.properties["DEVPATH"]:
            devPath = device.properties["DEVPATH"]
            break

    if devPath is None:
        raise USBFindError("Could not find serial device at USB location " + usb_phys_port)

    # we need the first 5 directories of the path.
    # for example if the result from udevadm is /devices/pci0000:00/0000:00:14.0/usb1/1-2/1-2:1.1/ttyUSB1/tty/ttyUSB1
    # then we want /devices/pci0000:00/0000:00:14.0/usb1/1-2/

    # easy trick to see what level of a USB-Hub the device is is to count the number of '.' in the port
    # (none indicates it's not part of a USB-Hub)
    usb_level = usb_phys_port.count(".")

    usb_dir = "/sys/" + "/".join(devPath.split("/")[1 : 6 + usb_level])
    # print(usb_dir)

    with open(usb_dir + "/devnum", "r") as f:
        dev_num = int(f.read())

    with open(usb_dir + "/busnum", "r") as f:
        bus_num = int(f.read())

    # print(bus_num, dev_num)
    return Path(f"/dev/bus/usb/{bus_num:03}/{dev_num:03}")
